/*
   ͻ
                     The Knight's Journey            May 10, 1990 
   ͹
              an example of a recursive function at work          
   ͹
                              By                                  
                         Rob Laltrello   <-- Prodigy ID BXMK05A   
                             -and-                                
                          Mark Pitely                             
   ͼ

           **************************************************
           **** PLEASE DISTRIBUTE UNMODIFIED COPIES ONLY ****
           **************************************************


     The Knight's Journey is provided "AS IS" without warranty of any
kind, either expressed or implied, including, but not limited to the
implied warranties of merchantability and fitness for any purpose.  The
entire risk as to the quality and performance of this program is with
the user and should the program prove defective, the user and not the
author will assume all responsibility for correcting all information.
The author does not warrant that this program will function in the
desired mode or will meet any user's requirements or that the operation
of the program will be error free.


     Since this is my first attempt at a C program, there are probably a
million ways to optimize this program.  This is not an original concept,
just an example how recursion can help you solve a problem.  In this
program, a chess knight begins in the center of the board and moves
exactly as a knight moves in the game of chess (two up, one over (or
visa versa)).  The board is solved when the knight has moved to every
space on the board without moving to any space twice. You can trace the
knight's moves sequentially starting from the number one and going
through the board size squared (i.e. a board size of 5 would be 1 through
25).  The knight continues attempting to move until it can move no longer
(i.e. it has already moved to that particular square). . . that is when
it must backtrack and try a new path to the solution.  It keeps
repeating this until the solution is found (see the "move" function).

     When the board is solved, an alarm will sound and the solution is
written to the file "SOLUTION.KNT" along with the time it took to
solve.

To start this program, type one of the following:

1)  C>knight

2)  C>knight boardsize
       (example  C>knight 5           <-- board size of 5)

3)  C>knight boardsize graphics
       (example  C>knight 14 yes      <-- board size of 14, graphics on)
       (example  C>knight 10 no       <-- board size of 10, graphics off)


     It might take you a while to figure out what the program is doing.
Just keep in mind how a knight moves and you should figure it out.

     If you want to solve (and not just watch) a board size that is
larger than 8, I would suggest doing so with the graphics off (it is
about twice as fast).

     You may edit this source code to any extent provided you only
distribute unmodified copies.


                                           Rob Laltrello
                                           Prodigy ID - BXMK05A

*/

#include <graphics.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

 static int         xmove[9] = {0,1,2,2,1,-1,-2,-2,-1};
 static int         ymove[9] = {0,2,1,-1,-2,-2,-1,1,2};

 int                boardsize=0;           /* Do not change this!! */

 int                board[15][15];         /* Must be one greater than*/
                                           /* maximum boardsize*/

 int                startx;                /* X starting location */
 int                starty;                /* Y starting location */
 int                dummy;

 int                moveorder[9];          /* This is the order in which the
                                              knight attempts his moves */

 int                curmove;               /* current move */
 int                done;
 int                xfudge, yfudge;
 long               start_time,stop_time,howlong;
 int                hour=0,min=0,sec=0;
 int                wantgraphics=1;

drawboard()
{
   int count;

   xfudge = (getmaxx() / boardsize) - 1;
   yfudge = (getmaxy() / boardsize) - 1;
   for (count = 1; count <= boardsize; count++)
      {
         line(count * xfudge,1,count * xfudge,boardsize * yfudge);
         line(1,count * yfudge,boardsize * xfudge,count * yfudge);
      }
   rectangle(0,0,boardsize * xfudge,boardsize * yfudge);
}


info()
{
   int ch;

   clrscr();
   printf("                         The Knight's Journey\n");
   printf("                                   By\n");
   printf("                     Rob Laltrello and Mark Pitely\n");
   printf("\n");
   printf("The object of this simulation is to have a single knight move to\n");
   printf("every location on the board moving just as the knight moves in\n");
   printf("chess and not moving to any location twice.\n");
   printf("\n");
   printf("You have selected a board size of %d.  For testing purposes, I\n",boardsize);
   printf("suggest you use a board size of 5 because it usually takes less\n");
   printf("than 5 minutes to solve on a 6 MHz IBM PC AT.\n");
   printf("\n");
   printf("A board size of 8 (standard chess board) can take up to 12 hours\n");
   printf("to solve on an IBM PC AT (maybe even longer!).\n");
   printf("\n");
   printf("I have never had the patience (or a fast enough computer) to solve\n");
   printf("a board size larger than 8.\n");
   printf("\n");
   printf("While the program is running, press ESC to abort.  For more\n");
   printf("information, or if you don\'t understand what is going on, see the\n");
   printf("source code\n");
   printf("\n");
   printf("An alarm will sound when the board is solved.\n");
   printf("\n");
   printf("Press ESC to quit or any other key to continue.");
   ch=getch();
   if (ch==27)
      {
         clrscr();
         printf("Program halted at user\'s request.\n");
         printf("\nCreated using Turbo C, Copyright (c) Borland 1987, 1988.\n");
         exit(1);
      }
   if (!wantgraphics)
      {
         clrscr();
         printf("Now attempting to solve a board size of %d...\n\n",boardsize);
         printf("The graphics are now off.  If you want to \"see\" what is happening\n");
         printf("you should restart and set graphics on.\n\n");
         printf("This could take a while... You might want to shut off your monitor and\n");
         printf("just listen for the alarm especially if you are solving a board size of\n");
         printf("8 or greater.\n\n");
         printf("Press ESC to abort.\n\n\n");
      }

}

getboardsize()
{
    clrscr();
    printf("If you are running this program for the first time, \n");
    printf("I recommend you select a board size of 5.\n\n");
    printf("Enter a board size (must be between 5 and 14) --> ");
    scanf("%d",&boardsize);
    check_boardsize();
}

getgraphics()
{
    char g_ans[3];

    printf("\nIf you are running this program for the first time,\n");
    printf("answer \"yes\" to the following question.\n\n");
    printf("Do you want the graphics (yes or no)? ");
    scanf("%s",g_ans);
    if ((strcmp(strlwr(g_ans),"no"))==0 || (strcmp(strlwr(g_ans),"n")==0 ))
        wantgraphics=0;

}

check_boardsize()
{

/****************************************************
*        boardsize must be between 5 and 14!!       *
****************************************************/

    if (boardsize>14 || boardsize<5)
       {
          sound(1000); delay(1000); nosound();
          printf("Board size must be between 5 and 14.\n");
          printf("Please enter a valid board size.\n\n");
          printf("Press any key to continue...\n");
          getch();
          getboardsize();
       }
}

initarray()
{
   int h,j,k,x,y;
   int notfound;

   startx       = boardsize / 2 + 1 ;  /* X starting location */
   starty       = boardsize / 2 + 1 ;  /* Y starting location */


   for (j = 1; j <= 8; j++)
      moveorder[j] = 0;

   notfound = 1;
   randomize();
   for (j = 1; j <= 8; j++)       /* This randomly sets up moveorder[] */
      {
          h = random(8);
          if (h == 0) h = 8;
          for (k = 1; k <= 8; k++)
            if (h == moveorder[k])
               {
                  if (j != 1) j = j - 1;
                  k = 8;
                  notfound = 0;
                }
         if (notfound)
            moveorder[j] = h;
         notfound = 1;
       }
   for (x = 1; x <= boardsize; x++)
      for (y = 1; y <= boardsize; y++)
         board[x][y] = 0;
   curmove = 1;

}


initialize()
{
   int g_error, gd, gm ;
   done = 0;
   if (registerbgidriver(EGAVGA_driver) <0) exit(1);  /* register EGAVGA */
   detectgraph(&gd,&gm);
   initgraph(&gd,&gm,"");
   g_error=graphresult();
   if (g_error<0)
      {
         clrscr();
         sound(1000); delay(1000); nosound();
         printf("\n\nThis program requires EGA or VGA graphics to use graphics mode.\n");
         exit(1);
      }
}




start_timer()
{
   time_t secs_now;

   start_time=time(&secs_now);
}

stop_timer()
{
   time_t secs_now;

   stop_time=time(&secs_now);
}

calc_time()
{
   howlong=stop_time-start_time;
   if (howlong>=60)
      {

         min=howlong / 60;  /* get full minutes             */
         sec=howlong % 60;  /* get leftover seconds         */
      }
   else
      sec=howlong;          /*If under 60, just get seconds */

   if (min>=60)
      {
         hour=min / 60;     /* get full hours               */
         min=min % 60;      /* get leftover minutes         */
      }
}

do_first()
{
   int x,y;
   char strmove[5];


   board[startx][starty] = 1;
   x = startx - 1;
   y = starty - 1;
   if (wantgraphics)
      {
         setfillstyle(1,1);
         bar(x * xfudge + 1,y * yfudge + 1,((x + 1) * xfudge) - 1,((y + 1) * yfudge) - 1);
         itoa(curmove,strmove,10);
         outtextxy((((((x + 1) * xfudge) - 1) - (x * xfudge + 1)) / 2) + (x * xfudge + 1)-5,
                   (((((y + 1) * yfudge) - 1) - (y * yfudge + 1)) / 2) + (y * yfudge + 1)-3,strmove);
      }
}




int checkmove(int x, int y, int count)
{
   int      cx, cy;

   dummy = moveorder[count];
   cx = x + xmove[dummy];
   cy = y + ymove[dummy];
   if ((cx > 0) && (cy > 0) && (cx < boardsize + 1) && (cy < boardsize + 1) && (board[cx][cy] == 0))
      return 1;
   else
      return 0;
}


/*********************************
* This is the recursive function *
*********************************/

move(int x, int y)

{
   int      tx, ty;
   int ch=0;
   int count;
   char strmove[5];


   count = 0;
   do
      {
         count+=1;
         if (checkmove(x,y,count))
            {
               board[x][y] = curmove;
               curmove+=1;
               dummy = moveorder[count];
               tx = x + xmove[dummy] - 1;
               ty = y + ymove[dummy] - 1;
               if (wantgraphics)
                  {
                     setfillstyle(1,1);
                     bar(tx * xfudge + 1,ty * yfudge + 1,((tx + 1) * xfudge) - 1,((ty + 1) * yfudge) - 1);
                     itoa(curmove,strmove,10);
                     outtextxy((((((tx + 1) * xfudge) - 1) - (tx * xfudge + 1)) / 2) + (tx * xfudge + 1)-5,
                               (((((ty + 1) * yfudge) - 1) - (ty * yfudge + 1)) / 2) + (ty * yfudge + 1)-3,strmove);
                  }
               if (curmove == (boardsize*boardsize))
                  done = 1;
               dummy = moveorder[count];
               move(x + xmove[dummy],y + ymove[dummy]);
               if (!done)
                  {
                     board[x][y] = 0;
                     curmove-=1;
                     if (wantgraphics)
                        {
                           setfillstyle(1,0);
                           bar(tx * xfudge + 1,ty * yfudge + 1,((tx + 1) * xfudge) - 1,((ty + 1) * yfudge) - 1);
                        }
                  }

            }
         if (kbhit())           /* Check to see if ESC has been hit */
            {
               ch=getch();
               if (ch==27)
                  {
                     stop_timer();
                     calc_time();
                     if (wantgraphics)
                        closegraph();
                     printf("Program halted at user\'s request.\n\n");
                     printf("Program ran for %d hour(s), %d minute(s) and %d second(s).\n",hour,min,sec);
                     printf("\nCreated using Turbo C, Copyright (c) Borland 1987, 1988.\n");

                     exit(1);
                  }
            }
      } while (!((count==8) || (done)));
}

write_file()
{
   FILE  *f;
   int x,y;

   f=fopen("SOLUTION.KNT","w+");
   if (f == NULL)
      {
        printf("Problem opening file");
        return;
      }
   for (y = 1; y <= boardsize; y++)
      {
         for (x = 1; x <= boardsize; x++)
            {
               if (board[x][y]==0)
                   board[x][y]=(boardsize*boardsize);
               fprintf(f,"%5d",board[x][y]);
            }
         fprintf(f,"\n");
      }
   fprintf(f,"\n");
   fprintf(f,"Solution took %d hour(s), %d minute(s) and %d second(s).\n",hour,min,sec);
   fclose(f);
}


main(int argc, char * argv[])
{
   if (argc>1)         /* Get 1st arg from command line (if it is there)*/
     {
        boardsize=atoi(argv[1]);
        check_boardsize();
        if (argc!=3)
           getgraphics();
     }
   if (argc>2)         /* Get 2nd arg from command line (if it is there)*/
     {
        if ((strcmp(strlwr(argv[2]),"no"))==0 || (strcmp(strlwr(argv[2]),"n")==0 ))
           wantgraphics=0;
        if ((strcmp(strlwr(argv[2]),"yes"))==0 || (strcmp(strlwr(argv[2]),"y")==0 ))
           wantgraphics=1;
     }

   if (boardsize==0)
      {
        getboardsize();
        getgraphics();
      }
   info();
   initarray();
   if (wantgraphics)
      {
         initialize();
         drawboard();
      }
   start_timer();
   do_first();
   move(startx,starty);
   stop_timer();
   sound(1500); delay(3000);nosound();  /* Sound alarm when board is solved*/
   calc_time();
   write_file();
   if (wantgraphics)
      getch();
      closegraph();
   printf("See file SOLUTION.KNT for a copy of the solution.\n");
   printf("Solution took %d hour(s), %d minute(s) and %d second(s).\n",hour,min,sec);
   printf("\nCreated using Turbo C, Copyright (c) Borland 1987, 1988.\n");
}

